import { Response } from './../../../response';
import { EnvironmentConfiguration, ObjectHelper } from 'dino-core';
import { NextFunction, Request } from 'express';
import { ClientHttpException } from '../../../exception/client.http.exception';
import { BeforeHandlerMiddleware } from '../../before.handler.middleware';

type SecurityConfiguration = {
  enabled: boolean;
  authenticator: string;
};

export class SecurityMiddleware extends BeforeHandlerMiddleware {
  private readonly configuration: SecurityConfiguration;

  public constructor(environment: EnvironmentConfiguration) {
    super();
    this.configuration = environment.getOrDefault('dino:security', { enabled: false });
  }
  /**
   * Definition of the function that will handle the security business logic associated with this middleware.
   *
   * @param {import('express').Request} req Express request instance
   * @param {import('express').Response} res express response instance
   * @param {Function} next the next function that will be invoked as part of the express chain
   */
  handle(req: Request, _res: Response, next: NextFunction): void {
    if (this.configuration.enabled) {
      const authorization: string | undefined = req.headers.authorization;
      if (ObjectHelper.isNotDefined(authorization)) {
        return next(ClientHttpException.create(401, 'Unauthorized'));
      }
    }
    next();
  }
}
