//    Copyright 2018 Quirino Brizi [quirino.brizi@gmail.com]
//
//    Licensed under the Apache License, Version 2.0 (the "License");
//    you may not use this file except in compliance with the License.
//    You may obtain a copy of the License at
//
//        http://www.apache.org/licenses/LICENSE-2.0
//
//    Unless required by applicable law or agreed to in writing, software
//    distributed under the License is distributed on an "AS IS" BASIS,
//    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//    See the License for the specific language governing permissions and
//    limitations under the License.

const { assert } = require('chai');
const { FilesystemDescriptorProvider } = require('../../../main/ts/openapi/filesystem.descriptor.provider');

describe('FilesystemDescriptorProvider', () => {
  it('Load default configuration if variable not defined', () => {
    let environment = new Map();
    let testObj = new FilesystemDescriptorProvider(environment);
    // act
    let actual = testObj.provide();
    // assert
    assert.equal(actual, 'src/main/resources/routes.yml');
  });

  it('Load configuration from : variable', () => {
    let environment = new Map();
    environment.set('dino:openapi:descriptor', 'path/with/colon');
    environment.set('dino_openapi_descriptor', 'path/with/underscore');
    environment.set('DINO_OPENAPI_DESCRIPTOR', 'path/with/underscore/uppercase');
    let testObj = new FilesystemDescriptorProvider(environment);
    // act
    let actual = testObj.provide();
    // assert
    assert.equal(actual, 'path/with/colon');
  });

  it('Load configuration from _ variable', () => {
    let environment = new Map();
    environment.set('dino_openapi_descriptor', 'path/with/underscore');
    environment.set('DINO_OPENAPI_DESCRIPTOR', 'path/with/underscore/uppercase');
    let testObj = new FilesystemDescriptorProvider(environment);
    // act
    let actual = testObj.provide();
    // assert
    assert.equal(actual, 'path/with/underscore');
  });

  it('Load configuration from uppercase _ variable', () => {
    let environment = new Map();
    environment.set('DINO_OPENAPI_DESCRIPTOR', 'path/with/underscore/uppercase');
    let testObj = new FilesystemDescriptorProvider(environment);
    // act
    let actual = testObj.provide();
    // assert
    assert.equal(actual, 'path/with/underscore/uppercase');
  });
});
